<?php

namespace Slovocast\Domain\Repository\Episode;

use DateTimeImmutable;
use DateTime;
use Slovocast\Domain\Entity\Episode;
use Slovocast\Domain\Entity\Channel;
use Slovocast\Domain\Repository\Episode\EpisodeRepositoryInterface;
use Slovocast\Exception\EntityNotFoundException;
use Slovocast\Infrastructure\Api\Database\DatabaseHandlerInterface;

class EpisodeRepository implements EpisodeRepositoryInterface
{
    public function __construct(
        private DatabaseHandlerInterface $db
    ) { }

    protected function getEpisodeFromResults(array $results): Episode
    {
        $results['createdAt'] = new DateTimeImmutable($results['created_at']);
        $results['updatedAt'] = new DateTime($results['updated_at']);

        return Episode::fromArray($results);
    }

    public function get(int $id): Episode
    {
        $query = "SELECT * FROM episodes WHERE id = :id";
        $statement = $this->db->getConnection()->query($query);    
        $results = $statement->fetch(\PDO::FETCH_ASSOC);

        if (!is_array($results) || !count($results)) {
            throw new EntityNotFoundException("Unable to find episode.");
        }

        $episode = $this->getEpisodeFromResults($results);
        return $episode;
    }

    public function getFromChannel(Channel $channel): array 
    {
        $query = "SELECT * FROM episodes WHERE channel_id = :channelId";
        $statement = $this->db->getConnection()->query($query);
        $results = $statement->fetch(\PDO::FETCH_ASSOC);

        if (!is_array($results) || !count($results)) {
            throw new EntityNotFoundException("Unable to find any episodes.");
        }

        return array_map(fn($r) => $this->getEpisodeFromResults($r), $results);
    }

    public function create(Episode $episode): Episode
    {
        $query = "INSERT INTO episodes (
            title, link, length, description, serialNumber,
            explicit, publishedDate, episodeType
        ) VALUE (
            :title, :link, :length, :description, :serialNumber,
            :explicit, :publishedDate, :episodeType
        )";

        $statement = $this->db->getConnection()->prepare($query);
        $results = $statement->execute([
            ':title' => $episode->getTitle(),
            ':link' => $episode->getLink(),
            ':length' => $episode->getLength(),
            ':description' => $episode->getDescription(),
            ':serialNumber' => $episode->getSerialNumber(),
            ':explicit' => $episode->isExplicit(),
            ':publishedDate' => $episode->getPublishedDate(),
            ':episodeType' => $episode->getEpisodeType(),
        ]);

        return $episode;
    }

    /**
     * @TODO Implement a plan for deleting
     */
    public function delete(Episode $episode): bool
    {
        return false;
    }
}
